/*  gnutrition - a nutrition and diet analysis program.
 *  Copyright (C) 2000 Edgar Denny (e.denny@ic.ac.uk)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

void
gnutr_show_recipe_win( void);

/* activate. */

void
on_recipe_win_open_activate( GtkMenuItem *, gpointer);

void
on_recipe_win_new_file_activate( GtkMenuItem *, gpointer);

void
on_recipe_win_exit_activate( GtkMenuItem *, gpointer);

void
on_recipe_win_add_food_activate( GtkMenuItem *, gpointer);

void
on_recipe_win_edit_food_activate( GtkMenuItem *, gpointer);

void
on_recipe_win_delete_food_activate( GtkMenuItem *, gpointer);

void
on_recipe_win_pref_activate( GtkMenuItem *, gpointer);

void
on_recipe_win_about_activate( GtkMenuItem *, gpointer);

void
on_recipe_win_manual_activate( GtkMenuItem *, gpointer);

void
on_recipe_win_save_activate( GtkMenuItem *, gpointer);

void
on_recipe_win_save_as_activate( GtkMenuItem *, gpointer);

/* button released. */

void
on_recipe_win_new_button_released( GtkButton *, gpointer);

void
on_recipe_win_save_button_released( GtkButton *, gpointer);

void
on_recipe_win_nutr_goal_button_released( GtkButton *, gpointer);

void
on_recipe_win_delete_button_released( GtkButton *, gpointer);

void
on_recipe_win_add_button_released( GtkButton *, gpointer);

void
on_recipe_win_edit_button_released( GtkButton *, gpointer);

void
on_recipe_win_nutr_comp_button_released( GtkButton *, gpointer);

void
on_recipe_win_open_button_released( GtkButton *, gpointer);

/* select row. */

void
on_recipe_win_ingredient_clist_select_row( GtkCList *, 
		gint, gint, GdkEvent *, gpointer);

/* button enter. */

void
on_recipe_win_new_button_enter( GtkButton *, gpointer);

void
on_recipe_win_open_button_enter( GtkButton *, gpointer);

void
on_recipe_win_save_button_enter( GtkButton *, gpointer);

void
on_recipe_win_add_button_enter( GtkButton *, gpointer);

void
on_recipe_win_delete_button_enter( GtkButton *, gpointer);

void
on_recipe_win_edit_button_enter( GtkButton *, gpointer);

void
on_recipe_win_nutr_goal_button_enter( GtkButton *, gpointer);

void
on_recipe_win_nutr_comp_button_enter( GtkButton *, gpointer);

/* button leave. */

void
on_recipe_win_new_button_leave( GtkButton *, gpointer);

void
on_recipe_win_open_button_leave( GtkButton *, gpointer);

void
on_recipe_win_save_button_leave( GtkButton *, gpointer);

void
on_recipe_win_add_button_leave( GtkButton *, gpointer);

void
on_recipe_win_delete_button_leave( GtkButton *, gpointer);

void
on_recipe_win_edit_button_leave( GtkButton *, gpointer);

void
on_recipe_win_nutr_comp_button_leave( GtkButton *, gpointer);

void
on_recipe_win_nutr_goal_button_leave( GtkButton *, gpointer);

/* extra. */

void
gnutr_free_recipe( void);

gboolean
gnutr_open_recipe_file( gchar *);

gboolean
gnutr_save_recipe_to_file( gchar *);

void
on_recipe_win_destroy( GtkWidget *, gpointer);

gint
on_recipe_win_delete_event( GtkWidget *, GdkEvent *, gpointer);

